#!/bin/sh
#
# Copyright (C) 2013  Paul Wolneykien <manowar@altlinux.org>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# General filename pattern with stating version part. Can be used
# to define more specific patterns as "$filename$BASIC_VERPAT".
BASIC_VERPAT='-\([^-]\+\).tar.[bgx]*z[0-9]*'

# Outputs the list of available versions for the given
# SourceForge.net hosted project.
#
# The project RSS URI can be obtained at the "Files" section
# on the project page. The filename regex, if passed, limits
# the files in the output list and should contain \(\) pair
# to select the version number. By default, the pattern is
# '-\([^-]\+\).tar.[bgx]*z'.
#
# The output format is: version [tab] URI
#
# args: sourceforge-rss-URI [filename-regex]
get_available_sourceforge_versions()
{
	local uri="$1"
    local pat="${2:-$BASIC_VERPAT}"

    pat="$(echo $pat | sed -e 's,/,\\/,g')"
    rsstail -1 -l -u "$uri" | sed -n -e "
/^Title:[[:space:]]\\(.*$pat\\)[[:space:]]*\$/ {
    s/^Title:[[:space:]]\\+\\(.*$pat\\)[[:space:]]*\$/\\2/;
    h; n;
    s/^Link:[[:space:]]\\+//;
    H; g;
    s/\\n/\\t/p;
}"
}

# Outputs the current gear package version (V-altV).
get_package_current_version()
{
        gear --describe 2>/dev/null | sed -e 's/^[^[:space:]]\+[[:space:]]\+\([^[:space:]]\+\)[[:space:]]\+\([^[:space:]]\+\)\?.*$/\1-\2/'
}

# Outputs the current gear package plain version.
get_package_plain_current_version()
{
	get_package_current_version | sed -e 's/-alt[0-9._][^-]*$//'
}

# Compares the specified package versions.
#
# args: ver-major-1 ver-minor-1 ver-major-2 ver-minor-2
compare_versions()
{
        local vernum1="$1"
        local rcnum1="$2"
        local vernum2="$3"
        local rcnum2="$4"

	if [ -z "$vernum1" ]; then
		echo -1
		return 0
	fi

        if [ "$vernum1$rcnum1" = "$vernum2$rcnum2" ]; then
                echo 0
        elif [ "$vernum1" == "$vernum2" ]; then
                if [ -z "$rcnum1" ]; then
                        echo 1
                elif [ -z "$rcnum2" ]; then
                        echo -1
                else
                        rpmvercmp "$rcnum1" "$rcnum2"
                fi
        else
                rpmvercmp "$vernum1" "$vernum2"
        fi
}

# Compares the specified package plain versions.
#
# args: ver-1 ver-2
compare_versions_plain()
{
	compare_versions "$1" '' "$2" ''
}

# Outputs the list of versions, each greater than the current
# or specified package version.
#
# input: The list of version uri pairs.
# args: [comparator] [current-version]
get_greater_source_versions()
{
	local cmp="${1-compare_versions_plain}"
    local v0="${2-$(get_package_plain_current_version)}"

    while IFS="$(printf '\t')" read -r v rest; do
		if [ $("$cmp" "$v0" "$v") -lt 0 ]; then
            printf '%s\t%s\n' "$v" "$rest" 2>/dev/null
        fi
	done
}

# Outputs the lowest available source package version, if any.
#
# input: The list of version uri pairs.
# args: [comparator] [current-version]
get_next_source_version()
{
	local cmp="${1-compare_versions_plain}"

	get_greater_source_versions "$@" | (
        least=
        uri=
        while IFS="$(printf '\t')" read -r v rest; do
            if [ -z "$least" ] || [ $("$cmp" "$v" "$least") -lt 0 ]; then
                least="$v"
                uri="$rest"
            fi
        done

        [ -n "$least" ] && printf '%s\t%s\n' "$least" "$uri" 2>/dev/null ||:
    )
}

# Outputs the highest (latest) available source package version,
# if any.
#
# input: The list of version uri pairs.
# args: [comparator] [current-version]
get_latest_source_version()
{
	local cmp="${1-compare_versions_plain}"

	get_greater_source_versions "$@" | (
        latest=
        uri=
        while IFS="$(printf '\t')" read -r v rest; do
            if [ -z "$latest" ] || [ "$("$cmp" "$v" "$latest")" = "1" ]; then
                latest="$v"
                uri="$rest"
            fi
        done

        [ -n "$latest" ] && printf '%s\t%s\n' "$latest" "$uri" 2>/dev/null ||:
    )
}

# Outputs the list of available versions for the given
# GitHub hosted project.
#
# The output format is: version [tab] URI
#
# args: github-user project
get_available_github_versions()
{
    curl -s "https://api.github.com/repos/$1/$2/tags" | python -c "
import sys
import json
tags = json.load (sys.stdin)
for tag in tags:
    print '%s\\t%s\\n' % (tag['name'], tag['tarball_url'])
"
}

# Outputs the list of available versions published as files
# at the given network directory.
#
# The filename regex, if passed, limits the files in
# the output list and should contain \(\) pair to select
# the version number. By default, the pattern is
# '-\([^-]\+\).tar.[bgx]*z'.
#
# The output format is: version [tab] URI
# args: directory-URI [filename-regex]
get_available_pubdir_versions()
{
	local uri="$1"
    local pat="${2:-$BASIC_VERPAT}"

    pat="$(echo $pat | sed -e 's,/,\\/,g')"
    local uriq="$(echo ${uri%/} | sed -e 's,/,\\/,g')"
    (
        if echo "$uri" | grep -iq '^ftp:'; then
            curl -sl "$uri"
        elif echo "$uri" | grep -iq '^https\?:'; then
            curl -s "$uri" | \
                sed -e "s/[[:space:]][Hh][Rr][Ee][Ff]=['\"][^'\"]*['\"]/\\n&\\n/g" | \
                sed -n -e "s/^[[:space:]][Hh][Rr][Ee][Ff]=['\"]\\([^'\"]*\\)['\"]/\\1/p"
        else
            echo "Unknown URI scheme: ${uri%%:*}" >&2
        fi
    ) | sed -n -e "s/^.*$pat\$/\\1\\t$uriq\\/&/p"
}

# Outputs the list of available versions for the named
# Python module using the information from PyPI.
#
# The output format is: version [tab] URI
#
# args: module-name
get_available_pypi_versions()
{
    local name="$1"
    local base_uri='https://pypi.python.org/packages/source/'

    get_available_pubdir_versions "$base_uri$(echo "$name" | cut -c1)/$name/" "$name$BASIC_VERPAT"
}
